/*------------------------------------------------------------------------------------------------------------
 Looping Louie  (Nur Vorwrtsflug)
 Realisiert mit Tiny25
 
 Interner Oszillator mit 8MHz, 5V Versorgungsspannung (4.3V an CPU)
 (Vorteiler /8 ausgeschaltet)
 
 Layout:
                      Reset  1    8    +5V
                        PB3  2     7    PB2: Led Eingang (Rauschgenerator)
            Taster      PB4  3     6    PB1: 
                        GND  4     5    PB0: PWM-Ausgang (FET)

 Input:
  PB4  Taster (GND=variable Geschwindigkeit freischalten)
Output:
  PB0  Ausgang zur Endstufe (FET irlu024N) 
*/

#define F_CPU 8000000UL	/* CPU clock in Hertz */

//#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


#define U8  unsigned char
#define U16 unsigned int
#define U32 unsigned long
#define I16 int
#define I32 long int

#define BIT0 1
#define BIT1 2
#define BIT2 4
#define BIT3 8
#define BIT4 16
#define BIT5 32
#define BIT6 64
#define BIT7 128


//Versorgung:5V 
//Ausgangsspannung: 51 steps/Volt
//Standard: 3V (2 Mignon-Zellen)
#define PWM_LANGSAM      118  //2.3V 
#define PWM_Mittel       153  //3.0V 
#define PWM_Schnell      184  //3.6V
#define PWM_Max          244  //4,8V

#define PWM_Standard      153 //Ca. 3V


//------------------------------------------------------------------------------------------------------------
U8 zufallsBit(void){  //Rauschen erfassen
  ADMUX=1; //ADC1/PB2 LED-Spannung, Uref=Vcc
  ADCSRA = (1<<ADEN) |  //Enable ADC
           (1<<ADSC) |  //Start conversion
           (1<<ADIF) |  //Clear(!) interrupt flag
           (1<<ADPS2)|
           (1<<ADPS1)    //ADC Prescaler=64 - gibt 125KHz
           ;
           
  //Warte, bis Messung fertig
  while( !( ADCSRA & (1<<ADIF) ) )
  ;  
  return ADCL & 1; //die oberen Bits interessieren nicht
}

//------------------------------------------------------------------------------------------------------------
U8 zufallsByte(void){
static U16 oldSeed=0;
U16 seed;
U8 i;

  if (0==oldSeed){
    for (i=0; i<15; i++) oldSeed+=oldSeed+zufallsBit(); //init 
  }

  seed=181*(oldSeed)+359;// 8 bit random number=high(SEED)
  oldSeed=seed;
  return seed>>8;
}
//------------------------------------------------------------------------------------------------------------
int main(void){

U8 stufe=1;
U8 wechsel=0;
U16 countdown=15;
U16 zufall=0;
U8 i;
U8 pwm=255;
U8 immer3v=1;

  cli();  //Dieses Programm luft ohne Interrupts
  
  DDRB = BIT0;  // PB0  Ausgang
  PORTB = BIT3 | BIT4; // Pull-Up auf Port PB3+PB4
 
  OCR0A  = 0;
  TCNT0  = 0;
  TCCR0A =  (1<<COM0A1) //PWM-Ausgang
           |(1<<WGM00)  //FAST PWM
           |(1<<WGM01)
           ;
  TCCR0B = (1<<CS00);   //Teiler 1:1 gibt 31.5KHz PWM (8MHz/256)
  
  //Sanft anfahren
  for (i=0; i<255; i++){
    OCR0A = i;
    _delay_ms(10); //Gesamtdauer 255*10ms = 2,55s
  }

  while(1){
    zufall = zufallsByte()+zufallsBit();

    countdown--;
    if (0==countdown){
      countdown=20 + zufall / 3;  //Gibt 2..10.5s
      wechsel=1;
    }
   if (immer3v) stufe=4; 
    switch(stufe){
      case 0: 
        pwm = PWM_LANGSAM;
        if (wechsel){
          stufe=(zufall & 3);
          if (0==stufe) stufe=1;  //Nicht in dieser Stufe bleiben
        }
        break;
      case 1: 
        pwm = PWM_Mittel;
        if (wechsel){
          stufe=(zufall & 3);
          if (1==stufe) stufe=2;
        }
        break;
      case 2: 
        pwm = PWM_Schnell;
        if (wechsel){
          stufe=(zufall & 3);
          if (2==stufe) stufe=3;
        }
        break;
      case 3: 
        pwm = PWM_Max;
        if (countdown > 60) countdown=60; //Nicht so lange rasen
        if (wechsel){
          stufe=(zufall & 3);
          if (3==stufe){
   	        if (PINB & BIT3){
              stufe=5; //Stop-Spurt
              OCR0A = PWM_Mittel;
            }else{
              stufe=0;
            }
          }
        }
        break;

      case 4: //Langweiler-Modus
        pwm = PWM_Standard;
        if (PINB & BIT4) stufe=(zufall & 3);
        break;

      case 5: //anhalten
        pwm = 0;
        if (0==OCR0A){
          countdown=6; 
          stufe=6;
        }
	      break;
      case 6: // Stop
        if (wechsel){
          pwm = 255;   //Vollgas
          OCR0A=255;
          countdown=111; 
          stufe=7;
	      }
        break;	
      case 7: //Spurt
        if (wechsel){
          stufe=(zufall & 3);
          if (3==stufe) stufe=1;
        }
        break;	
    } //switch
    wechsel=0; 
    for (i=0; i<10; i++){
      if (pwm > OCR0A) OCR0A=OCR0A+1;
      if (pwm < OCR0A) OCR0A=OCR0A-1;
      _delay_ms(10);  
       if (!(PINB & BIT4)){ //Taster gedrueckt
         immer3v=0;
         stufe=3; 
         countdown=40;
       }
         
    }
  }//While(1)
  return 0;
}
// EOF----------------------------------------------------------------------------------------------------------
